/**
 * @author Hamadou BA
 * Depot GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp1-ps-bibliotheque-de-calcul-matriciel.git
 * 
 * La classe Vecteur represente un vecteur
 * permettant de manipuler et effectuer des calculs sur des vecteurs
 * Cette classe herite de la classe Matrice et fournit des fonctionnalites comme l'affichage, 
 * la lecture a partir de fichiers, la modification de coefficients, et le calcul du produit scalaire
 */

package AlgLin;

import java.io.*;
import java.util.*;

public class Vecteur extends Matrice {

    /**
     * Constructeur avec un parametre entier reprsentant le nombre de lignes
     * Initialise un vecteur de taille nbligne avec une seule colonne
     * 
     * @param nbligne Le nombre de lignes du vecteur
     */
    Vecteur(int nbligne) {
        super(nbligne, 1);
    }

    /**
     * Constructeur avec un tableau de coefficients
     * Cree un vecteur a partir des coefficients fournis dans le tableau
     * 
     * @param coefficients Tableau de coefficients pour initialiser le vecteur
     */
    public Vecteur(double[] coefficients) {
        super(coefficients.length, 1);
        for (int i = 0; i < coefficients.length; i++) {
            this.coefficient[i][0] = coefficients[i];
        }
    }

    /**
     * Constructeur avec un fichier. 
     * Le fichier doit contenir un entier representant la taille du vecteur,
     * suivi des coefficients du vecteur, un par ligne
     * 
     * @param fichier Le chemin vers le fichier contenant les coefficients du vecteur
     */
    public Vecteur(String fichier) {
        super(0, 0);
        try {
            Scanner sc = new Scanner(new File(fichier));
            int taille = sc.nextInt();
            this.coefficient = new double[taille][1];
            for (int i = 0; i < taille; i++) {
                this.coefficient[i][0] = sc.nextDouble();
            }
            sc.close();
        } catch (FileNotFoundException e) {
            System.out.println("Fichier absent");
        } catch (Exception e) {
            System.out.println("Erreur lors de la lecture du fichier :");
        }
    }

    /**
     * Methode qui renvoie la taille du vecteur
     * 
     * @return La taille du vecteur
     */
    public int taille() {
        return this.coefficient.length;
    }

    /**
     * Methode qui renvoie le coefficient a une position donnee
     * 
     * @param position La position du coefficient a recuperer
     * @return Le coefficient a la position specifiee
     */
    
    public double getCoefficient(int position) {
        return this.coefficient[position][0];
    }

    /**
     * Methode pour modifier un coefficient a une position donnee.
     * 
     * @param position La position du coefficient a modifier
     * @param valeur   La nouvelle valeur du coefficient
     */
    public void setCoefficient(int position, double valeur) {
        this.coefficient[position][0] = valeur;
    }

    /**
     * Methode statique pour calculer le produit scalaire de deux vecteurs sans verification prealable.
     * 
     * @param v1 Le premier vecteur
     * @param v2 Le second vecteur
     * @return Le produit scalaire des deux vecteurs
     */
    public static double produitScalaire(Vecteur v1, Vecteur v2) {
        double produit = 0;
        for (int i = 0; i < v1.taille(); i++) {
            produit += v1.getCoefficient(i) * v2.getCoefficient(i);
        }
        return produit;
    }

    /**
     * Methode statique pour calculer le produit scalaire de deux vecteurs avec verification prealable.
     * Verifie que les vecteurs ont la même taille avant de calculer le produit scalaire.
     * 
     * @param v1 Le premier vecteur
     * @param v2 Le second vecteur
     * @return Le produit scalaire des deux vecteurs
     * @throws Exception Si les vecteurs n'ont pas la même taille
     */
    public static double verifProduitScalaire(Vecteur v1, Vecteur v2) throws Exception {
        if (v1.taille() != v2.taille()) {
            throw new Exception("Les vecteurs n'ont pas la même taille pour un produit scalaire.");
        }

        double produit = 0;
        for (int i = 0; i < v1.taille(); i++) {
            produit += v1.getCoefficient(i) * v2.getCoefficient(i);
        }

        return produit;
    }

    /**
     * Methode toString pour afficher le vecteur
     * 
     * @return La representation sous forme de chaine du vecteur
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("Vecteur : [");
        for (int i = 0; i < this.taille(); i++) {
            sb.append(this.coefficient[i][0]);
            if (i < this.taille() - 1) sb.append(", ");
        }
        sb.append("]");
        return sb.toString();
    }

    /**
     * Methode principale pour tester la classe Vecteur
     * @param args
     * @throws Exception Si une erreur se produit lors du calcul du produit scalaire
     */
    public static void main(String[] args) throws Exception {

        // Construction d'un vecteur a partir d'un tableau
        double[] tableau1 = {1.0, 2.0, 3.0};
        Vecteur v1 = new Vecteur(tableau1);
        System.out.println("Construction d'un vecteur par affectation d'un tableau : \n");
        System.out.println(v1);
        System.out.println(); 
        
        // Construction d'un vecteur a partir d'un fichier
        Vecteur v2 = new Vecteur("src/AlgLin/vecteur1.txt");
        System.out.println("Construction d'un vecteur par lecture d'un fichier :");
        System.out.println(v2);
        System.out.println();

        // Construction d'un vecteur de taille donnee (initialise a zero)
        Vecteur v3 = new Vecteur(5);
        System.out.println("Construction d'un vecteur de taille 5 :");
        System.out.println(v3);
        System.out.println();

        // Verification de la taille du vecteur
        System.out.println("Taille du vecteur v1 : " + v1.taille());
        System.out.println(); // Retour a la ligne pour separer les sections

        // Accès et affichage d'un coefficient specifique
        System.out.println("Coefficient a la position 1 dans le vecteur v1 : " + v1.getCoefficient(1));
        System.out.println(); // Retour a la ligne pour separer les sections

        // Modification d'un coefficient dans le vecteur
        System.out.println("Modification du coefficient a la position 1 de v1 a la valeur 10.0");
        v1.setCoefficient(1, 10.0);
        System.out.println("Vecteur v1 après modification :");
        System.out.println(v1);
        System.out.println();

        // Produit scalaire de deux vecteurs compatibles
        Vecteur v4 = new Vecteur(new double[]{4.0, 5.0, 6.0});
        double produit = Vecteur.produitScalaire(v1, v4);
        System.out.println("Produit scalaire de v1 et v4 :");
        System.out.println(produit);
        System.out.println();

        // Verification du produit scalaire avec verification de la compatibilite des tailles
        double produitVerifie = Vecteur.verifProduitScalaire(v1, v4);
        System.out.println("Produit scalaire de v1 et v4 (avec verification) :");
        System.out.println(produitVerifie);
    }
}
